<?php
// $Id$

/**
 * @file
 * Implements the Custom_Solr_Query class, extending Solr_Base_Query and providing
 * extra functionality for our searches.
 */
include_once(drupal_get_path('module', 'apachesolr') .'/Solr_Base_Query.php');

class Custom_Add_Solr_Query extends Solr_Base_Query {
  
  /**
   * Returns a default list of sorts.
   */
  protected function default_sorts() {
    // The array keys must always be real Solr index fields.
    $sorts = parent::default_sorts();
  
    $sorts['ss_feed_source'] = array('title' => t('Feed Source'), 'default' => 'asc');
    $sorts['ss_url'] = array('title' => t('Source URL'), 'default' => 'asc');
    
    return $sorts;
  }
  
  /**
   * Extract all uses of one named field from a filter string e.g. 'type:book'
   */
  public function filter_extract(&$filterstring, $name) {
    $extracted = array();
    // Range queries.  The "TO" is case-sensitive.
    $patterns[] = '/(^| |-)'. $name .':([\[\{](\S+) TO (\S+)[\]\}])/';
    // Match quoted values.
    $patterns[] = '/(^| |-)'. $name .':"([^"]*)"/';
    // Match (# OR #)
    $patterns[] = '/(^| |-)'. $name .':(\([^\(]*\))/';
    // Match unquoted values.
    $patterns[] = '/(^| |-)'. $name .':([^ ]*)/';

    foreach ($patterns as $p) {
      if (preg_match_all($p, $filterstring, $matches, PREG_SET_ORDER)) {
        foreach($matches as $match) {
          $filter = array();
          $filter['#query'] = $match[0];
          $filter['#exclude'] = ($match[1] == '-');
          $filter['#value'] = trim($match[2]);
          if (isset($match[3])) {
            // Extra data for range queries
            $filter['#start'] = $match[3];
            $filter['#end'] = $match[4];
          }
          $extracted[] = $filter;
          // Update the local copy of $filters by removing the match.
          $filterstring = str_replace($match[0], '', $filterstring);
        }
      }
    }
    return $extracted;
  }
  
  /**
   * Takes an array $field and combines the #name and #value in a way
   * suitable for use in a Solr query.
   */
  public function make_filter(array $filter) {
    // If the field value has spaces, or : in it, wrap it in double quotes.
    // unless it is a range query or OR-ed.
    if (preg_match('/[ :]/', $filter['#value'])
        && !isset($filter['#start'])
        && !preg_match('/[\[\{]\S+ TO \S+[\]\}]/', $filter['#value'])
        && !preg_match('/\(\S+ OR \S+( OR \S+)*\)/', $filter['#value'])) {
      $filter['#value'] = '"'. $filter['#value']. '"';
    }
    $prefix = empty($filter['#exclude']) ? '' : '-';
    return $prefix . $filter['#name'] . ':' . $filter['#value'];
  }
  
  public function has_filter($name, $value) {
    foreach ($this->fields as $pos => $values) {
      if (isset($values['#name']) && isset($values['#value']) && $values['#name'] == $name && ($values['#value'] == $value || preg_match('/[\s\(]'. $value .'[\s\)]/', $values['#value']))) {
        return TRUE;
      }
    }
    return FALSE;
  }
  
  /**
   * Remove filters from this query.
   */
  public function remove_filter($name, $value = NULL) {
    // We can only remove named fields.
    if (empty($name)) {
      return;
    }
    if (!isset($value)) {
      foreach ($this->fields as $pos => $values) {
        if ($values['#name'] == $name) {
          unset($this->fields[$pos]);
        }
      }
    }
    else {
      foreach ($this->fields as $pos => $values) {
        if ($values['#name'] == $name && $values['#value'] == $value) {
          unset($this->fields[$pos]);
        }
        else if ($values['#name'] == $name && preg_match('/[\s\(]'. $value .'[\s\)]/', $values['#value'])) {
          $matches = array();
          preg_match_all('/\d+/', $values['#value'], $matches);
          $new_values = array();
          foreach($matches[0] as $match_value) {
            if ($match_value != $value) {
              $new_values[] = $match_value;
            }
          }
          if (count($new_values) == 1) {
            $this->fields[$pos]['#value'] = $new_values[0];
          }
          else {
            $this->fields[$pos]['#value'] = '('. implode(' OR ', $new_values) .')';
          }
        }
      }
    }
  }
  
  /**
   * Parse the filter string in $this->filters into $this->fields.
   *
   * Builds an array of field name/value pairs.
   */
  /*protected function parse_filters() {
    $this->fields = array();
    $filterstring = $this->filterstring;

    // Gets information about the fields already in solr index.
    $index_fields = $this->solr->getFields();

    foreach ((array) $index_fields as $name => $data) {
      // Look for a field alias.
      $alias = isset($this->field_map[$name]) ? $this->field_map[$name] : $name;
      // Get the values for $name
      $extracted = $this->filter_extract($filterstring, $alias);
      if (count($extracted)) {
        if ($name == 'tid') {
          $pos = strpos($this->filterstring, $extracted[0]['#query']);
          $filter['#name'] = $name;
          $filter = $extracted[0];
          if (count($extracted) > 1) {
            for($index = 1; $index < count($extracted); $index++) {
              $filter['#value'] .= ' OR '. $extracted[$index]['#value'];
              $filter['#query'] .= ' OR '. $extracted[$index]['#value'];
            }
            $filter['#value'] = '('. $filter['#value'] .')';
            $filter['#query'] = 'tid:'. $filter['#value'];
          }
        }
        else {
          foreach ($extracted as $filter) {
            $pos = strpos($this->filterstring, $filter['#query']);
            // $solr_keys and $solr_crumbs are keyed on $pos so that query order
            // is maintained. This is important for breadcrumbs.
            $filter['#name'] = $name;
            $this->fields[$pos] = $filter;
          }
        }
      }
    }
    // Even though the array has the right keys they are likely in the wrong
    // order. ksort() sorts the array by key while maintaining the key.
    ksort($this->fields);
  }*/
  
  
  /**
   * Builds a set of filter queries from $this->fields and all subqueries.
   *
   * Returns an array of strings that can be combined into
   * a URL query parameter or passed to Solr as fq paramters.
   */
  protected function rebuild_fq($aliases = FALSE) {
    $fq = array();
    $filter_array = array();
    $fields = array();
    if ($aliases == FALSE) {
      foreach ($this->fields as $pos => $field) {
        // Look for a field alias.
        if ($aliases && isset($this->field_map[$field['#name']])) {
          $field['#name'] = $this->field_map[$field['#name']];
        }
        $filter_array[$field['#name']][] = $this->make_filter($field);
        
      }
      //dpm('rebuild_fq');
      //dpm($filter_array);
      foreach ($filter_array as $name => $filters) {
        if (count($filters) > 1) {
          $fq[] = $name .':('. implode(" OR ", preg_replace('/[^:]*:(.*)/', '\1', $filters)) .')';
          //$fq[] = "(" . implode(" OR ", $filters) . ")";
        }
        else if (count($filters) == 1) {
          $fq[] = $filters[0];
        }
      }
    }
    else {
      foreach ($this->fields as $pos => $field) {
        // Look for a field alias.
        if ($aliases && isset($this->field_map[$field['#name']])) {
          $field['#name'] = $this->field_map[$field['#name']];
        }
        $filter_array[$field['#name']][] = $this->make_filter($field);
      }
      //dpm('rebuild_fq');
      //dpm($filter_array);
      foreach ($filter_array as $name => $filters) {
        if (count($filters) > 1) {
          $fq[] = $name .':('. implode(" OR ", preg_replace('/[^:]*:(.*)/', '\1', $filters)) .')';
          //$fq[] = "(" . implode(" OR ",$filters) . ")";
        }
        else if (count($filters) == 1) {
          $fq[] = $filters[0];
        }
      }
    }
    foreach ($this->subqueries as $id => $data) {
      $subfq = $data['#query']->rebuild_fq($aliases);
      if ($subfq) {
        $operator = $data['#fq_operator'];
        $fq[] = implode(" {$operator} ", $subfq) ;
      }
    }
    return $fq;
  }

  protected function rebuild_query() {
    $query = $this->keys;
    foreach ($this->subqueries as $id => $data) {
      $operator = $data['#q_operator'];
      $subquery = $data['#query']->get_query_basic();
      if ($subquery) {
        $query .= " {$operator} ({$subquery})";
      }
    }
    return $query;
  }
}