<?php
// $Id$

/**
 * @file
 * Implements the Custom_Solr_Query class, extending Solr_Base_Query and providing
 * extra functionality for our searches - particularly allowing for multiple
 * dimensions in searches.
 */
include_once(drupal_get_path('module', 'apachesolr') .'/Solr_Base_Query.php');

class Custom_Mult_Solr_Query extends Solr_Base_Query {
  
  /**
   * Returns a default list of sorts.
   */
  protected function default_sorts() {
    // The array keys must always be real Solr index fields.
    $sorts = parent::default_sorts();
  
    $sorts['ss_feed_source'] = array('title' => t('Feed Source'), 'default' => 'asc');
    $sorts['ss_url'] = array('title' => t('Source URL'), 'default' => 'asc');
    
    return $sorts;
  }
  
  /**
   * Allows us to define multiple sort sets.
   */
  public function set_solrsort_multiple($name, $direction, $empty = FALSE) {
    if ($empty) {
      $this->solrsort = array();
    }
    if (isset($this->available_sorts[$name])) {
      $this->solrsort[] = array('#name' => $name, '#direction' => $direction);
    }
  }
  
  /**
   * Parses a sort string like 'type asc, title asc' so that it can be used.
   */
  protected function parse_sortstring() {
    // Substitute any field aliases with real field names.
    $sortstring = strtr($this->sortstring, array_flip($this->field_map));
    // Score is a special case - it's the default sort for Solr.
    if ('' == $sortstring) {
      $this->set_solrsort('score', 'asc');
    }
    else {
      // Validate and set sort parameter
      $this->set_solrsort_multiple(NULL, NULL, TRUE);
      $fields = implode('|', array_keys($this->available_sorts));
      if (preg_match_all('/(?:('. $fields .') (asc|desc),?)/',
                         $sortstring,
                         $matches,
                         PREG_SET_ORDER)) {
        foreach ($matches as $set_match) {
          $this->set_solrsort_multiple($set_match[1], $set_match[2]);
        }
      }
    }
  }
  
  /**
   * this may be called by other code, so we need to give it the format it's expecting.
   */
  public function get_solrsort() {
    if (is_array($this->solrsort) && count($this->solrsort)) {
      if (array_key_exists('#name', $this->solrsort)) {
        // There is only one sort pair, so we can return it.
        return $this->solrsort; // this is the default data.
      }
      $return = array('#name' => '', '#direction' => '');
      foreach($this->solrsort as $index => $sort) {
        if (!$index) {
          $return['#name'] = $sort['#name'];
          $return['#direction'] = $sort['#direction'];
        }
        else {
          $return['#direction'] = $return['#direction'] .','.
            $sort['#name'] .' '. $sort['#direction'];
        }
      }
      return $return;
    }
    return array('#name' => 'score', '#direction' => 'asc'); // default!
  }
  
  /**
   * Gets the solr sort keys in multiple format.
   */
  public function get_solrsort_multiple() {
    if (array_key_exists('#name', $this->solrsort)) {
      return array($this->solrsort); 
    }
    return $this->solrsort;
  }
  
  
}